"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var rx_1 = require("rx");
var fill_type_1 = require("../enums/fill-type");
var fitness_objective_1 = require("../enums/fitness-objective");
var mutate_type_1 = require("../enums/mutate-type");
var reproduce_type_1 = require("../enums/reproduce-type");
var index_1 = require("../index");
var fill_random_1 = require("../operators/fill-random");
var fill_worst_1 = require("../operators/fill-worst");
var worst_1 = require("../operators/worst");
var NaturalSelection = (function () {
    function NaturalSelection(popOptions, genOptions, create, fitness) {
        this.popOptions = popOptions;
        this.genOptions = genOptions;
        this.create = create;
        this.fitness = fitness;
        this.genomes = index_1.generateGenomes(this.popOptions.populationSize, this.genOptions);
    }
    NaturalSelection.prototype.reproduce = function (gens) {
        switch (this.popOptions.reproduceOptions.type) {
            case reproduce_type_1.ReproduceType.safeSampled:
                gens = index_1.safeSampledReproduceManyToMany(this.genomes, this.genomes.length, this.fitness, this.popOptions.objective, this.popOptions.reproduceOptions.sampleSize);
                break;
            case reproduce_type_1.ReproduceType.safe:
                gens = index_1.safeReproduceManyToMany(this.genomes, this.genomes.length, this.fitness, this.popOptions.objective);
                break;
            case reproduce_type_1.ReproduceType.sampled:
                gens = index_1.sampledReproduceManyToMany(this.genomes, this.genomes.length, this.fitness, this.popOptions.reproduceOptions.sampleSize);
                break;
            case reproduce_type_1.ReproduceType.normal:
                gens = index_1.reproduceManyToMany(this.genomes, this.genomes.length);
                break;
            default:
                gens = index_1.safeReproduceManyToMany(this.genomes, this.genomes.length, this.fitness, this.popOptions.objective);
                break;
        }
        return gens;
    };
    NaturalSelection.prototype.mutate = function (gens) {
        switch (this.popOptions.mutateOptions.type) {
            case mutate_type_1.MutateType.safeSampled:
                return index_1.safeSampledMutateMany(this.genomes, this.fitness, this.popOptions.objective, this.popOptions.mutateOptions.mutateChance, this.popOptions.mutateOptions.mutateOp, this.popOptions.mutateOptions.sampleSize);
            case mutate_type_1.MutateType.safe:
                return index_1.safeMutateMany(this.genomes, this.fitness, this.popOptions.objective, this.popOptions.mutateOptions.mutateChance, this.popOptions.mutateOptions.mutateOp);
            case mutate_type_1.MutateType.sampled:
                return index_1.sampledMutateMany(this.genomes, this.fitness, this.popOptions.objective, this.popOptions.mutateOptions.mutateChance, this.popOptions.mutateOptions.mutateOp, this.popOptions.mutateOptions.sampleSize);
            case mutate_type_1.MutateType.normal:
                return index_1.mutateMany(this.genomes, this.popOptions.mutateOptions.mutateChance, this.popOptions.mutateOptions.mutateOp);
            default:
                return index_1.safeMutateMany(this.genomes, this.fitness, this.popOptions.mutateOptions.mutateChance, this.popOptions.mutateOptions.mutateOp);
        }
    };
    NaturalSelection.prototype.fill = function (gens) {
        switch (this.popOptions.fillType) {
            case fill_type_1.FillType.worst:
                return fill_worst_1.fillWorst(gens, this.fitness, this.popOptions.fillPercent);
            case fill_type_1.FillType.random:
                return fill_random_1.fillRandom(gens, this.popOptions.fillPercent);
            case fill_type_1.FillType.none:
                return gens;
            default:
                return fill_worst_1.fillWorst(gens, this.fitness, this.popOptions.fillPercent);
        }
    };
    NaturalSelection.prototype.evolveStep = function () {
        var gens;
        gens = this.reproduce(this.genomes);
        gens = this.mutate(gens);
        gens = this.fill(gens);
        return gens;
    };
    NaturalSelection.prototype.evolve = function (generations) {
        var _this = this;
        _.range(generations).forEach(function (i) {
            _this.genomes = _this.evolveStep();
        });
        var b = index_1.best(this.genomes, this.fitness);
        return b;
    };
    NaturalSelection.prototype.evolve$ = function (generations, timeout) {
        var _this = this;
        if (generations === void 0) { generations = 1000; }
        if (timeout === void 0) { timeout = 3000; }
        return rx_1.Observable
            .interval(1)
            .map(function (i) { return _this.genomes; })
            .map(function (gens) {
            _this.genomes = _this.evolveStep();
            return _this.genomes;
        })
            .map(function (gens) {
            switch (_this.popOptions.objective) {
                case fitness_objective_1.FitnessObjective.maximize:
                    return index_1.best(gens, _this.fitness);
                case fitness_objective_1.FitnessObjective.minimize:
                    return worst_1.worst(gens, _this.fitness);
            }
        })
            .take(generations);
    };
    return NaturalSelection;
}());
exports.NaturalSelection = NaturalSelection;
//# sourceMappingURL=natural-selection.js.map